import json
import os

import requests

if 'GITLAB_ACCESS_TOKEN' not in os.environ:
    raise Exception('GITLAB_ACCESS_TOKEN required')

headers = {'Private-Token': os.environ['GITLAB_ACCESS_TOKEN']}


def trigger_pipeline(project_id, branch='master'):
    url = 'https://gitlab.com/api/v4/projects/%s/pipeline?ref=%s' % (project_id, branch)
    res = requests.post(url=url, headers=headers)
    res_dict = json.loads(res.text)
    return res_dict['id'], res_dict['web_url']


def get_pipeline(project_id, pipeline_id):
    url = 'https://gitlab.com/api/v4/projects/%s/pipelines/%s' % (project_id, pipeline_id)
    res = requests.get(url=url, headers=headers)
    return json.loads(res.text)

def get_jobs(project_id, pipeline_id):
    url = 'https://gitlab.com/api/v4/projects/%s/pipelines/%s/jobs' % (project_id, pipeline_id)
    res = requests.get(url=url, headers=headers)
    return json.loads(res.text)



from cerberus import Validator

# gitlab model

pipeline_statuses = [ 'created',
                      'waiting_for_resource',
                      'preparing',
                      'pending',
                      'running',
                      'success',
                      'failed',
                      'canceled',
                      'skipped',
                      'manual',
                      'scheduled', ]

job_statuses = [ 'created',
                 'pending',
                 'running',
                 'failed',
                 'success',
                 'canceled',
                 'skipped',
                 'manual', ]

environment_statuses = [ 'available', 'stopped' ]

# schemas

trigger_schema = { 'project_id': { 'type': 'integer', 'required': True },
                   'branch'    : { 'type': 'string' }, }

assertions_schema = { 'pipeline_status': { 'type': 'string',
                                           'allowed': pipeline_statuses,
                                           'required': True },
                      'job_count': { 'type': 'integer' },
                      'job_status': { 'type': 'dict',
                                      'keysrules': { 'type': 'string' },
                                      'valuesrules': { 'type': 'string',
                                                       'allowed': job_statuses } },
                      'artifact_count': { 'type': 'integer' },
                      'environment_status': { 'type': 'dict',
                                              'keysrules': { 'type': 'string' },
                                              'valuesrules': { 'type': 'string',
                                                               'allowed': environment_statuses },
                                            },
                    }

test_schema = { 'trigger': { 'type': 'dict', 'schema': trigger_schema, },
                'assertions': { 'type': 'dict', 'schema': assertions_schema, }, }

def validate_test(test):
    validator = Validator()
    return validator.validate(test, test_schema), validator.errors
